/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2015 SEGGER Microcontroller GmbH & Co. KG         *
*                                                                    *
*     Internet: segger.com   Support: support_embos@segger.com       *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 4.14                                             *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : OS_Bootloader.c
Purpose : embOS sample program for bootloader
--------- END-OF-HEADER ----------------------------------------------
*/

#include "RTOS.h"
#include "BSP.h"
#include "MK66F18.h"  // Device specific header file, contains CMSIS

#define THUMB_BIT         1
#define APP_START_ADDR    (0x00100000)
#define APP_STACK_PTR     (*(volatile OS_U32*)(APP_START_ADDR + 0x00)) 
#define APP_RESET_PTR     (*(volatile OS_U32*)(APP_START_ADDR + 0x04))

static OS_STACKPTR int StackHP[128], StackLP[128];   /* Task stacks */
static OS_TASK         TCBHP, TCBLP;         /* Task-control-blocks */
void (*AppPtr)(void);

static void HPTask(void) {
  while (1) {
    BSP_ToggleLED(0);
    //
    // Start the application
    //
    OS_IncDI();
    AppPtr    = (void (*)(void))(APP_RESET_PTR | THUMB_BIT);
    SCB->VTOR = APP_START_ADDR;
    __set_MSP(APP_STACK_PTR);  // Set main stack pointer to application initial stack value
    __set_CONTROL(0);          // Use MSP and Privileged in thread mode 
    AppPtr();                  // Start the application, we will not return from this function
  }
}

static void LPTask(void) {
  while (1) {
    OS_Delay(200);
  }
}

/*********************************************************************
*
*       main()
*/
int main(void) {
  OS_IncDI();                      /* Initially disable interrupts  */
  OS_InitKern();                   /* Initialize OS                 */
  OS_InitHW();                     /* Initialize Hardware for OS    */
  BSP_Init();                      /* Initialize LED ports          */
  /* You need to create at least one task before calling OS_Start() */
  OS_CREATETASK(&TCBHP, "HP Task", HPTask, 100, StackHP);
  OS_CREATETASK(&TCBLP, "LP Task", LPTask,  50, StackLP);
  OS_Start();                      /* Start multitasking            */
  return 0;
}

/****** End Of File *************************************************/
